@@ -1,27 +1,30 @@
 /*****************************************************************************
- *  $Id: munge.c 705 2007-02-04 07:52:37Z dun $
+ *  $Id: munge.c 771 2010-03-02 23:14:07Z dun $
  *****************************************************************************
- *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Produced at Lawrence Livermore National Laboratory.
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
+ *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
  *  For details, see <http://home.gna.org/munge/>.
  *
- *  This is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  MUNGE is free software: you can redistribute it and/or modify it under
+ *  the terms of the GNU General Public License as published by the Free
+ *  Software Foundation, either version 3 of the License, or (at your option)
+ *  any later version.  Additionally for the MUNGE library (libmunge), you
+ *  can redistribute it and/or modify it under the terms of the GNU Lesser
+ *  General Public License as published by the Free Software Foundation,
+ *  either version 3 of the License, or (at your option) any later version.
  *
- *  This is distributed in the hope that it will be useful, but WITHOUT
+ *  MUNGE is distributed in the hope that it will be useful, but WITHOUT
  *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- *  for more details.
+ *  and GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  and GNU Lesser General Public License along with MUNGE.  If not, see
+ *  <http://www.gnu.org/licenses/>.
  *****************************************************************************/
 
 
@@ -51,29 +54,29 @@
  *  Command-Line Options
  *****************************************************************************/
 
-#include <getopt.h>
-struct option opt_table[] = {
-    { "help",         0, NULL, 'h' },
-    { "license",      0, NULL, 'L' },
-    { "version",      0, NULL, 'V' },
-    { "no-input",     0, NULL, 'n' },
-    { "string",       1, NULL, 's' },
-    { "input",        1, NULL, 'i' },
-    { "output",       1, NULL, 'o' },
-    { "cipher",       1, NULL, 'c' },
-    { "list-ciphers", 0, NULL, 'C' },
-    { "mac",          1, NULL, 'm' },
-    { "list-macs",    0, NULL, 'M' },
-    { "zip",          1, NULL, 'z' },
-    { "list-zips",    0, NULL, 'Z' },
-    { "restrict-uid", 1, NULL, 'u' },
-    { "restrict-gid", 1, NULL, 'g' },
-    { "ttl",          1, NULL, 't' },
-    { "socket",       1, NULL, 'S' },
-    {  NULL,          0, NULL,  0  }
-};
+const char * const short_opts = ":hLVns:i:o:c:Cm:Mz:Zu:g:t:S:";
 
-const char * const opt_string = "hLVns:i:o:c:Cm:Mz:Zu:g:t:S:";
+#include <getopt.h>
+struct option long_opts[] = {
+    { "help",         no_argument,       NULL, 'h' },
+    { "license",      no_argument,       NULL, 'L' },
+    { "version",      no_argument,       NULL, 'V' },
+    { "no-input",     no_argument,       NULL, 'n' },
+    { "string",       required_argument, NULL, 's' },
+    { "input",        required_argument, NULL, 'i' },
+    { "output",       required_argument, NULL, 'o' },
+    { "cipher",       required_argument, NULL, 'c' },
+    { "list-ciphers", no_argument,       NULL, 'C' },
+    { "mac",          required_argument, NULL, 'm' },
+    { "list-macs",    no_argument,       NULL, 'M' },
+    { "zip",          required_argument, NULL, 'z' },
+    { "list-zips",    no_argument,       NULL, 'Z' },
+    { "restrict-uid", required_argument, NULL, 'u' },
+    { "restrict-gid", required_argument, NULL, 'g' },
+    { "ttl",          required_argument, NULL, 't' },
+    { "socket",       required_argument, NULL, 'S' },
+    {  NULL,          0,                 NULL,  0  }
+};
 
 
 /*****************************************************************************
@@ -238,7 +241,7 @@ parse_cmdline (conf_t conf, int argc, char **argv)
 
     for (;;) {
 
-        c = getopt_long (argc, argv, opt_string, opt_table, NULL);
+        c = getopt_long (argc, argv, short_opts, long_opts, NULL);
 
         if (c == -1) {                  /* reached end of option list */
             break;
@@ -327,10 +330,12 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                     i = pw_ptr->pw_uid;
                 }
                 else {
+                    errno = 0;
                     l = strtol (optarg, &p, 10);
-                    if ((optarg == p) || (*p != '\0')
-                            || (l < 0) || (l > INT_MAX)
-                            || ((l == LONG_MAX) && (errno == ERANGE))) {
+                    if (((errno == ERANGE)
+                                && ((l == LONG_MIN) || (l == LONG_MAX)))
+                            || (optarg == p) || (*p != '\0')
+                            || (l < 0) || (l > INT_MAX)) {
                         log_err (EMUNGE_SNAFU, LOG_ERR,
                             "Unrecognized user \"%s\"", optarg);
                     }
@@ -348,10 +353,12 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                     i = gr_ptr->gr_gid;
                 }
                 else {
+                    errno = 0;
                     l = strtol (optarg, &p, 10);
-                    if ((optarg == p) || (*p != '\0')
-                            || (l < 0) || (l > INT_MAX)
-                            || ((l == LONG_MAX) && (errno == ERANGE))) {
+                    if (((errno == ERANGE)
+                                && ((l == LONG_MIN) || (l == LONG_MAX)))
+                            || (optarg == p) || (*p != '\0')
+                            || (l < 0) || (l > INT_MAX)) {
                         log_err (EMUNGE_SNAFU, LOG_ERR,
                             "Unrecognized group \"%s\"", optarg);
                     }
@@ -365,12 +372,13 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 }
                 break;
             case 't':
+                errno = 0;
                 l = strtol (optarg, &p, 10);
                 if ((optarg == p) || (*p != '\0')) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
                         "Invalid time-to-live '%s'", optarg);
                 }
-                if (((l == LONG_MAX) && (errno == ERANGE)) || (l > INT_MAX)) {
+                if (((errno == ERANGE) && (l == LONG_MAX)) || (l > INT_MAX)) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
                         "Exceeded maximum time-to-live of %d seconds",
                         INT_MAX);
@@ -398,14 +406,41 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                     log_err (EMUNGE_SNAFU, LOG_ERR,
                         "Invalid option \"-%c\"", optopt);
                 }
-                else {
+                else if (optind > 1) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
                         "Invalid option \"%s\"", argv[optind - 1]);
                 }
+                else {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Unable to process command-line");
+                }
+                break;
+            case ':':
+                if ((optind > 1)
+                        && (strncmp (argv[optind - 1], "--", 2) == 0)) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Missing argument for option \"%s\"",
+                        argv[optind - 1]);
+                }
+                else if (optopt > 0) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Missing argument for option \"-%c\"", optopt);
+                }
+                else {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Unable to process command-line");
+                }
                 break;
             default:
-                log_err (EMUNGE_SNAFU, LOG_ERR,
-                    "Unimplemented option \"%s\"", argv[optind - 1]);
+                if ((optind > 1)
+                        && (strncmp (argv[optind - 1], "--", 2) == 0)) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Unimplemented option \"%s\"", argv[optind - 1]);
+                }
+                else {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Unimplemented option \"-%c\"", c);
+                }
                 break;
         }
     }
