@@ -1,27 +1,30 @@
 /*****************************************************************************
- *  $Id: unmunge.c 705 2007-02-04 07:52:37Z dun $
+ *  $Id: unmunge.c 771 2010-03-02 23:14:07Z dun $
  *****************************************************************************
- *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Produced at Lawrence Livermore National Laboratory.
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
+ *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
  *  For details, see <http://home.gna.org/munge/>.
  *
- *  This is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  MUNGE is free software: you can redistribute it and/or modify it under
+ *  the terms of the GNU General Public License as published by the Free
+ *  Software Foundation, either version 3 of the License, or (at your option)
+ *  any later version.  Additionally for the MUNGE library (libmunge), you
+ *  can redistribute it and/or modify it under the terms of the GNU Lesser
+ *  General Public License as published by the Free Software Foundation,
+ *  either version 3 of the License, or (at your option) any later version.
  *
- *  This is distributed in the hope that it will be useful, but WITHOUT
+ *  MUNGE is distributed in the hope that it will be useful, but WITHOUT
  *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- *  for more details.
+ *  and GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  and GNU Lesser General Public License along with MUNGE.  If not, see
+ *  <http://www.gnu.org/licenses/>.
  *****************************************************************************/
 
 
@@ -31,7 +34,6 @@
 
 #include <sys/types.h>                  /* include before in.h for bsd       */
 #include <netinet/in.h>                 /* include before inet.h for bsd     */
-#include <arpa/inet.h>                  /* for inet_ntop()                   */
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -49,7 +51,7 @@
 #include <munge.h>
 #include "common.h"
 #include "license.h"
-#include "missing.h"
+#include "missing.h"                    /* for inet_ntop()                   */
 #include "read.h"
 #include "version.h"
 
@@ -65,22 +67,22 @@
  *  Command-Line Options
  *****************************************************************************/
 
-#include <getopt.h>
-struct option opt_table[] = {
-    { "help",         0, NULL, 'h' },
-    { "license",      0, NULL, 'L' },
-    { "version",      0, NULL, 'V' },
-    { "input",        1, NULL, 'i' },
-    { "no-output",    0, NULL, 'n' },
-    { "metadata",     1, NULL, 'm' },
-    { "output",       1, NULL, 'o' },
-    { "keys",         1, NULL, 'k' },
-    { "list-keys",    0, NULL, 'K' },
-    { "socket",       1, NULL, 'S' },
-    {  NULL,          0, NULL,  0  }
-};
+const char * const short_opts = ":hLVi:nm:o:k:KS:";
 
-const char * const opt_string = "hLVi:nm:o:k:KS:";
+#include <getopt.h>
+struct option long_opts[] = {
+    { "help",      no_argument,       NULL, 'h' },
+    { "license",   no_argument,       NULL, 'L' },
+    { "version",   no_argument,       NULL, 'V' },
+    { "input",     required_argument, NULL, 'i' },
+    { "no-output", no_argument,       NULL, 'n' },
+    { "metadata",  required_argument, NULL, 'm' },
+    { "output",    required_argument, NULL, 'o' },
+    { "keys",      required_argument, NULL, 'k' },
+    { "list-keys", no_argument,       NULL, 'K' },
+    { "socket",    required_argument, NULL, 'S' },
+    {  NULL,       0,                 NULL,  0  }
+};
 
 
 /*****************************************************************************
@@ -327,7 +329,7 @@ parse_cmdline (conf_t conf, int argc, char **argv)
 
     for (;;) {
 
-        c = getopt_long (argc, argv, opt_string, opt_table, NULL);
+        c = getopt_long (argc, argv, short_opts, long_opts, NULL);
 
         if (c == -1) {                  /* reached end of option list */
             break;
@@ -379,14 +381,41 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                     log_err (EMUNGE_SNAFU, LOG_ERR,
                         "Invalid option \"-%c\"", optopt);
                 }
-                else {
+                else if (optind > 1) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
                         "Invalid option \"%s\"", argv[optind - 1]);
                 }
+                else {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Unable to process command-line");
+                }
+                break;
+            case ':':
+                if ((optind > 1)
+                        && (strncmp (argv[optind - 1], "--", 2) == 0)) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Missing argument for option \"%s\"",
+                        argv[optind - 1]);
+                }
+                else if (optopt > 0) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Missing argument for option \"-%c\"", optopt);
+                }
+                else {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Unable to process command-line");
+                }
                 break;
             default:
-                log_err (EMUNGE_SNAFU, LOG_ERR,
-                    "Unimplemented option \"%s\"", argv[optind - 1]);
+                if ((optind > 1)
+                        && (strncmp (argv[optind - 1], "--", 2) == 0)) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Unimplemented option \"%s\"", argv[optind - 1]);
+                }
+                else {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Unimplemented option \"-%c\"", c);
+                }
                 break;
         }
     }
