@@ -1,27 +1,30 @@
 /*****************************************************************************
- *  $Id: auth_recv.c 705 2007-02-04 07:52:37Z dun $
+ *  $Id: auth_recv.c 776 2010-03-11 01:06:43Z dun $
  *****************************************************************************
- *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Produced at Lawrence Livermore National Laboratory.
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
+ *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
  *  For details, see <http://home.gna.org/munge/>.
  *
- *  This is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  MUNGE is free software: you can redistribute it and/or modify it under
+ *  the terms of the GNU General Public License as published by the Free
+ *  Software Foundation, either version 3 of the License, or (at your option)
+ *  any later version.  Additionally for the MUNGE library (libmunge), you
+ *  can redistribute it and/or modify it under the terms of the GNU Lesser
+ *  General Public License as published by the Free Software Foundation,
+ *  either version 3 of the License, or (at your option) any later version.
  *
- *  This is distributed in the hope that it will be useful, but WITHOUT
+ *  MUNGE is distributed in the hope that it will be useful, but WITHOUT
  *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- *  for more details.
+ *  and GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  and GNU Lesser General Public License along with MUNGE.  If not, see
+ *  <http://www.gnu.org/licenses/>.
  *****************************************************************************/
 
 
@@ -34,7 +37,6 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <munge.h>
-#include "auth_policy.h"
 #include "log.h"
 #include "m_msg.h"
 #include "path.h"
@@ -50,12 +52,12 @@ static void _check_auth_client_dir (const char *dir, int got_force);
 void
 auth_recv_init (const char *srvrdir, const char *clntdir, int got_force)
 {
-#ifdef MUNGE_AUTH_RECVFD_MKNOD
+#ifdef AUTH_METHOD_RECVFD_MKNOD
     if (geteuid () != 0) {
         log_err (EMUNGE_SNAFU, LOG_ERR,
             "MUNGE daemon requires root privileges");
     }
-#endif /* MUNGE_AUTH_RECVFD_MKNOD */
+#endif /* AUTH_METHOD_RECVFD_MKNOD */
 
     _check_auth_server_dir (srvrdir, got_force);
     _check_auth_client_dir (clntdir, got_force);
@@ -66,7 +68,7 @@ auth_recv_init (const char *srvrdir, const char *clntdir, int got_force)
 static void
 _check_auth_server_dir (const char *dir, int got_force)
 {
-#ifdef MUNGE_AUTH_RECVFD
+#if defined(AUTH_METHOD_RECVFD_MKFIFO) || defined(AUTH_METHOD_RECVFD_MKNOD)
     int          got_symlink;
     struct stat  st;
     int          n;
@@ -143,14 +145,14 @@ _check_auth_server_dir (const char *dir, int got_force)
         log_msg (LOG_WARNING,
             "The auth server dir is inaccessible: %s", ebuf);
     }
-#endif /* MUNGE_AUTH_RECVFD */
+#endif /* AUTH_METHOD_RECVFD_MKFIFO || AUTH_METHOD_RECVFD_MKNOD */
     return;
 }
 
 static void
 _check_auth_client_dir (const char *dir, int got_force)
 {
-#ifdef MUNGE_AUTH_RECVFD
+#if defined(AUTH_METHOD_RECVFD_MKFIFO) || defined(AUTH_METHOD_RECVFD_MKNOD)
     int          got_symlink;
     struct stat  st;
     int          n;
@@ -228,7 +230,7 @@ _check_auth_client_dir (const char *dir, int got_force)
         log_msg (LOG_WARNING,
             "The auth client dir is inaccessible: %s", ebuf);
     }
-#endif /* MUNGE_AUTH_RECVFD */
+#endif /* AUTH_METHOD_RECVFD_MKFIFO || AUTH_METHOD_RECVFD_MKNOD */
     return;
 }
 
@@ -237,7 +239,7 @@ _check_auth_client_dir (const char *dir, int got_force)
  *  getpeereid
  *****************************************************************************/
 
-#ifdef MUNGE_AUTH_GETPEEREID
+#ifdef AUTH_METHOD_GETPEEREID
 
 #include <sys/types.h>
 
@@ -251,14 +253,14 @@ auth_recv (m_msg_t m, uid_t *uid, gid_t *gid)
     return (0);
 }
 
-#endif /* MUNGE_AUTH_GETPEEREID */
+#endif /* AUTH_METHOD_GETPEEREID */
 
 
 /*****************************************************************************
  *  getpeerucred
  *****************************************************************************/
 
-#ifdef MUNGE_AUTH_GETPEERUCRED
+#ifdef AUTH_METHOD_GETPEERUCRED
 
 #include <ucred.h>
 
@@ -291,14 +293,14 @@ auth_recv (m_msg_t m, uid_t *uid, gid_t *gid)
     return (rc);
 }
 
-#endif /* MUNGE_AUTH_GETPEERUCRED */
+#endif /* AUTH_METHOD_GETPEERUCRED */
 
 
 /*****************************************************************************
- *  peercred sockopt
+ *  SO_PEERCRED sockopt
  *****************************************************************************/
 
-#ifdef MUNGE_AUTH_PEERCRED
+#ifdef AUTH_METHOD_SO_PEERCRED
 
 #include <sys/socket.h>
 
@@ -321,14 +323,52 @@ auth_recv (m_msg_t m, uid_t *uid, gid_t *gid)
     return (0);
 }
 
-#endif /* MUNGE_AUTH_PEERCRED */
+#endif /* AUTH_METHOD_SO_PEERCRED */
+
+
+/*****************************************************************************
+ *  LOCAL_PEERCRED sockopt
+ *****************************************************************************/
+
+#ifdef AUTH_METHOD_LOCAL_PEERCRED
+
+#include <sys/param.h>                  /* for FreeBSD */
+#include <sys/socket.h>
+#include <sys/ucred.h>
+#include <sys/un.h>                     /* for FreeBSD */
+
+#ifndef HAVE_SOCKLEN_T
+typedef int socklen_t;                  /* socklen_t is uint32_t in Posix.1g */
+#endif /* !HAVE_SOCKLEN_T */
+
+int
+auth_recv (m_msg_t m, uid_t *uid, gid_t *gid)
+{
+    struct xucred cred;
+    socklen_t len = sizeof (cred);
+
+    if (getsockopt (m->sd, 0, LOCAL_PEERCRED, &cred, &len) < 0) {
+        log_msg (LOG_ERR, "Unable to get peer identity: %s", strerror (errno));
+        return (-1);
+    }
+    if (cred.cr_version != XUCRED_VERSION) {
+        log_msg (LOG_ERR, "Unable to get peer identity: invalid xucred v%d",
+            cred.cr_version);
+        return (-1);
+    }
+    *uid = cred.cr_uid;
+    *gid = cred.cr_gid;
+    return (0);
+}
+
+#endif /* AUTH_METHOD_LOCAL_PEERCRED */
 
 
 /*****************************************************************************
  *  strrecvfd struct (mkfifo)
  *****************************************************************************/
 
-#ifdef MUNGE_AUTH_RECVFD_MKFIFO
+#ifdef AUTH_METHOD_RECVFD_MKFIFO
 
 #include <assert.h>
 #include <fcntl.h>                      /* open, O_RDONLY */
@@ -411,14 +451,14 @@ err:
     return (-1);
 }
 
-#endif /* MUNGE_AUTH_RECVFD_MKFIFO */
+#endif /* AUTH_METHOD_RECVFD_MKFIFO */
 
 
 /*****************************************************************************
  *  strrecvfd struct (mknod)
  *****************************************************************************/
 
-#ifdef MUNGE_AUTH_RECVFD_MKNOD
+#ifdef AUTH_METHOD_RECVFD_MKNOD
 
 #include <assert.h>
 #include <fcntl.h>                      /* open, O_RDWR */
@@ -593,14 +633,14 @@ _s_pipe (int fd[2])
     return (0);
 }
 
-#endif /* MUNGE_AUTH_RECVFD_MKNOD */
+#endif /* AUTH_METHOD_RECVFD_MKNOD */
 
 
 /*****************************************************************************
  *  strrecvfd struct (common)
  *****************************************************************************/
 
-#ifdef MUNGE_AUTH_RECVFD
+#if defined(AUTH_METHOD_RECVFD_MKFIFO) || defined(AUTH_METHOD_RECVFD_MKNOD)
 
 #include <assert.h>
 #include <stdio.h>                      /* snprintf */
@@ -712,4 +752,4 @@ end:
     return (e == EMUNGE_SUCCESS ? 0 : -1);
 }
 
-#endif /* MUNGE_AUTH_RECVFD */
+#endif /* AUTH_METHOD_RECVFD_MKFIFO || AUTH_METHOD_RECVFD_MKNOD */
