@@ -1,27 +1,30 @@
 /*****************************************************************************
- *  $Id: conf.c 705 2007-02-04 07:52:37Z dun $
+ *  $Id: conf.c 771 2010-03-02 23:14:07Z dun $
  *****************************************************************************
- *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Produced at Lawrence Livermore National Laboratory.
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
+ *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
  *  For details, see <http://home.gna.org/munge/>.
  *
- *  This is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  MUNGE is free software: you can redistribute it and/or modify it under
+ *  the terms of the GNU General Public License as published by the Free
+ *  Software Foundation, either version 3 of the License, or (at your option)
+ *  any later version.  Additionally for the MUNGE library (libmunge), you
+ *  can redistribute it and/or modify it under the terms of the GNU Lesser
+ *  General Public License as published by the Free Software Foundation,
+ *  either version 3 of the License, or (at your option) any later version.
  *
- *  This is distributed in the hope that it will be useful, but WITHOUT
+ *  MUNGE is distributed in the hope that it will be useful, but WITHOUT
  *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- *  for more details.
+ *  and GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  and GNU Lesser General Public License along with MUNGE.  If not, see
+ *  <http://www.gnu.org/licenses/>.
  *****************************************************************************/
 
 
@@ -31,7 +34,6 @@
 
 #include <sys/types.h>                  /* include before in.h for bsd */
 #include <netinet/in.h>                 /* include before inet.h for bsd */
-#include <arpa/inet.h>                  /* for inet_ntop() */
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
@@ -43,12 +45,11 @@
 #include <sys/socket.h>                 /* for AF_INET */
 #include <sys/stat.h>
 #include <unistd.h>
-#include "auth_policy.h"
 #include "conf.h"
 #include "license.h"
 #include "log.h"
 #include "md.h"
-#include "missing.h"
+#include "missing.h"                    /* for inet_ntop() */
 #include "munge_defs.h"
 #include "path.h"
 #include "str.h"
@@ -60,27 +61,27 @@
  *  Command-Line Options
  *****************************************************************************/
 
-#  include <getopt.h>
-struct option opt_table[] = {
-    { "help",       0, NULL, 'h' },
-    { "license",    0, NULL, 'L' },
-    { "version",    0, NULL, 'V' },
-    { "force",      0, NULL, 'f' },
-    { "foreground", 0, NULL, 'F' },
-    { "socket",     1, NULL, 'S' },
-    { "advice",     0, NULL, 'A' },
-    { "key-file",          1, NULL, '0' },
-    { "num-threads",       1, NULL, '1' },
-#ifdef MUNGE_AUTH_RECVFD
-    { "auth-server-dir",   1, NULL, '2' },
-    { "auth-client-dir",   1, NULL, '3' },
-#endif /* MUNGE_AUTH_RECVFD */
-    { "group-check-mtime", 1, NULL, '4' },
-    { "group-update-time", 1, NULL, '5' },
-    {  NULL,        0, NULL,  0  }
-};
+const char * const short_opts = ":hLVfFS:";
 
-const char * const opt_string = "hLVfFS:";
+#include <getopt.h>
+struct option long_opts[] = {
+    { "help",              no_argument,       NULL, 'h' },
+    { "license",           no_argument,       NULL, 'L' },
+    { "version",           no_argument,       NULL, 'V' },
+    { "force",             no_argument,       NULL, 'f' },
+    { "foreground",        no_argument,       NULL, 'F' },
+    { "socket",            required_argument, NULL, 'S' },
+    { "advice",            no_argument,       NULL, 'A' },
+    { "key-file",          required_argument, NULL, '0' },
+    { "num-threads",       required_argument, NULL, '1' },
+#if defined(AUTH_METHOD_RECVFD_MKFIFO) || defined(AUTH_METHOD_RECVFD_MKNOD)
+    { "auth-server-dir",   required_argument, NULL, '2' },
+    { "auth-client-dir",   required_argument, NULL, '3' },
+#endif /* AUTH_METHOD_RECVFD_MKFIFO || AUTH_METHOD_RECVFD_MKNOD */
+    { "group-check-mtime", required_argument, NULL, '4' },
+    { "group-update-time", required_argument, NULL, '5' },
+    {  NULL,               0,                 NULL,  0  }
+};
 
 
 /*****************************************************************************
@@ -150,7 +151,7 @@ create_conf (void)
     conf->gids_interval = MUNGE_GROUP_UPDATE_TIMER;
     conf->nthreads = MUNGE_THREADS;
 
-#ifdef MUNGE_AUTH_RECVFD
+#if defined(AUTH_METHOD_RECVFD_MKFIFO) || defined(AUTH_METHOD_RECVFD_MKNOD)
 
     if (!(conf->auth_server_dir = strdup (MUNGE_AUTH_SERVER_DIR))) {
         log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
@@ -160,7 +161,7 @@ create_conf (void)
         log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
             "Cannot dup auth-client-dir default string");
     }
-#endif /* MUNGE_AUTH_RECVFD */
+#endif /* AUTH_METHOD_RECVFD_MKFIFO || AUTH_METHOD_RECVFD_MKNOD */
 
     conf->auth_rnd_bytes = MUNGE_AUTH_RND_BYTES;
 
@@ -237,7 +238,7 @@ parse_cmdline (conf_t conf, int argc, char **argv)
 
     for (;;) {
 
-        c = getopt_long (argc, argv, opt_string, opt_table, NULL);
+        c = getopt_long (argc, argv, short_opts, long_opts, NULL);
 
         if (c == -1) {                  /* reached end of option list */
             break;
@@ -280,15 +281,17 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                         "Cannot dup key-file name string");
                 break;
             case '1':
+                errno = 0;
                 l = strtol (optarg, &p, 10);
-                if ((optarg == p) || (*p != '\0')
+                if (((errno == ERANGE) && ((l == LONG_MIN) || (l == LONG_MAX)))
+                        || (optarg == p) || (*p != '\0')
                         || (l <= 0) || (l > INT_MAX)) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
                         "Invalid value \"%s\" for num-threads", optarg);
                 }
                 conf->nthreads = l;
                 break;
-#ifdef MUNGE_AUTH_RECVFD
+#if defined(AUTH_METHOD_RECVFD_MKFIFO) || defined(AUTH_METHOD_RECVFD_MKNOD)
             case '2':
                 if (conf->auth_server_dir)
                     free (conf->auth_server_dir);
@@ -303,18 +306,22 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                     log_errno (EMUNGE_NO_MEMORY, LOG_ERR,
                         "Cannot dup auth-client-dir cmdline string");
                 break;
-#endif /* MUNGE_AUTH_RECVFD */
+#endif /* AUTH_METHOD_RECVFD_MKFIFO || AUTH_METHOD_RECVFD_MKNOD */
             case '4':
+                errno = 0;
                 l = strtol (optarg, &p, 10);
-                if ((optarg == p) || (*p != '\0')) {
+                if (((errno == ERANGE) && ((l == LONG_MIN) || (l == LONG_MAX)))
+                        || (optarg == p) || (*p != '\0')) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
                         "Invalid value \"%s\" for group-check-mtime", optarg);
                 }
                 conf->got_group_stat = !! l;
                 break;
             case '5':
+                errno = 0;
                 l = strtol (optarg, &p, 10);
-                if ((optarg == p) || (*p != '\0')
+                if (((errno == ERANGE) && ((l == LONG_MIN) || (l == LONG_MAX)))
+                        || (optarg == p) || (*p != '\0')
                         || (l < INT_MIN) || (l > INT_MAX)) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
                         "Invalid value \"%s\" for group-update-time", optarg);
@@ -323,16 +330,45 @@ parse_cmdline (conf_t conf, int argc, char **argv)
                 break;
             /* End deprecated cmdline opts */
             case '?':
-                if (optopt > 0)
+                if (optopt > 0) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
                         "Invalid option \"-%c\"", optopt);
-                else
+                }
+                else if (optind > 1) {
                     log_err (EMUNGE_SNAFU, LOG_ERR,
                         "Invalid option \"%s\"", argv[optind - 1]);
+                }
+                else {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Unable to process command-line");
+                }
+                break;
+            case ':':
+                if ((optind > 1)
+                        && (strncmp (argv[optind - 1], "--", 2) == 0)) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Missing argument for option \"%s\"",
+                        argv[optind - 1]);
+                }
+                else if (optopt > 0) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Missing argument for option \"-%c\"", optopt);
+                }
+                else {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Unable to process command-line");
+                }
                 break;
             default:
-                log_err (EMUNGE_SNAFU, LOG_ERR,
-                    "Unimplemented option \"%s\"", argv[optind - 1]);
+                if ((optind > 1)
+                        && (strncmp (argv[optind - 1], "--", 2) == 0)) {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Unimplemented option \"%s\"", argv[optind - 1]);
+                }
+                else {
+                    log_err (EMUNGE_SNAFU, LOG_ERR,
+                        "Unimplemented option \"-%c\"", c);
+                }
                 break;
         }
     }
@@ -380,13 +416,13 @@ display_help (char *prog)
      */
     printf ("\n");
 
-#ifdef MUNGE_AUTH_RECVFD
+#if defined(AUTH_METHOD_RECVFD_MKFIFO) || defined(AUTH_METHOD_RECVFD_MKNOD)
     printf ("  %*s %s [%s]\n", w, "--auth-server-dir=DIR",
             "Specify auth-server directory", MUNGE_AUTH_SERVER_DIR);
 
     printf ("  %*s %s [%s]\n", w, "--auth-client-dir=DIR",
             "Specify auth-client directory", MUNGE_AUTH_CLIENT_DIR);
-#endif /* MUNGE_AUTH_RECVFD */
+#endif /* AUTH_METHOD_RECVFD_MKFIFO || AUTH_METHOD_RECVFD_MKNOD */
 
     printf ("  %*s Specify whether to check \"%s\" mtime [%d]\n",
             w, "--group-check-mtime=BOOL", GIDS_GROUP_FILE,
