@@ -1,27 +1,30 @@
 /*****************************************************************************
- *  $Id: gids.c 705 2007-02-04 07:52:37Z dun $
+ *  $Id: gids.c 771 2010-03-02 23:14:07Z dun $
  *****************************************************************************
- *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Produced at Lawrence Livermore National Laboratory.
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
+ *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
  *  For details, see <http://home.gna.org/munge/>.
  *
- *  This is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  MUNGE is free software: you can redistribute it and/or modify it under
+ *  the terms of the GNU General Public License as published by the Free
+ *  Software Foundation, either version 3 of the License, or (at your option)
+ *  any later version.  Additionally for the MUNGE library (libmunge), you
+ *  can redistribute it and/or modify it under the terms of the GNU Lesser
+ *  General Public License as published by the Free Software Foundation,
+ *  either version 3 of the License, or (at your option) any later version.
  *
- *  This is distributed in the hope that it will be useful, but WITHOUT
+ *  MUNGE is distributed in the hope that it will be useful, but WITHOUT
  *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- *  for more details.
+ *  and GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  and GNU Lesser General Public License along with MUNGE.  If not, see
+ *  <http://www.gnu.org/licenses/>.
  *****************************************************************************
  *  Refer to "gids.h" for documentation on public functions.
  *****************************************************************************/
@@ -110,7 +113,8 @@ typedef struct gids_uid  * gids_uid_t;
 
 static void         _gids_update (gids_t gids);
 static hash_t       _gids_hash_create (void);
-static int          _gids_user_to_uid (hash_t hash, char *user);
+static int          _gids_user_to_uid (
+                        hash_t uid_hash, char *user, uid_t *uid_p);
 static int          _gids_hash_add (hash_t hash, uid_t uid, gid_t gid);
 static gids_gid_t   _gids_head_alloc (uid_t uid);
 static void         _gids_head_del (gids_gid_t g);
@@ -284,7 +288,7 @@ _gids_update (gids_t gids)
     time_t          t_now;
     struct stat     st;
     int             do_update = 1;
-    hash_t          hash, hash_bak;
+    hash_t          hash;
 
     assert (gids != NULL);
 
@@ -304,28 +308,30 @@ _gids_update (gids_t gids)
             t_now = st.st_mtime;
         }
     }
-    if (do_update && (hash = _gids_hash_create ())) {
+    /*  Update the GIDS mapping.
+     */
+    hash = NULL;
+    if (do_update) {
+        hash = _gids_hash_create ();
+    }
+    if ((errno = pthread_mutex_lock (&gids->lock)) != 0) {
+        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to lock gids mutex");
+    }
+    /*  Replace the old GIDS mapping if the update was successful.
+     */
+    if (hash) {
+
+        hash_t hash_bak;
 
-        if ((errno = pthread_mutex_lock (&gids->lock)) != 0) {
-            log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to lock gids mutex");
-        }
         hash_bak = gids->hash;
         gids->hash = hash;
-
-        if ((errno = pthread_mutex_unlock (&gids->lock)) != 0) {
-            log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to unlock gids mutex");
-        }
-        hash_destroy (hash_bak);
+        hash = hash_bak;
         t_last_update = t_now;
     }
     /*  Enable subsequent updating of the GIDs mapping only if the update
      *    interval is positive.
      */
-    if ((errno = pthread_mutex_lock (&gids->lock)) != 0) {
-        log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to lock gids mutex");
-    }
     gids->timer = 0;
-
     if (gids->interval > 0) {
         gids->timer = timer_set_relative (
                 (callback_f) _gids_update, gids, gids->interval * 1000);
@@ -337,6 +343,11 @@ _gids_update (gids_t gids)
     if ((errno = pthread_mutex_unlock (&gids->lock)) != 0) {
         log_errno (EMUNGE_SNAFU, LOG_ERR, "Unable to unlock gids mutex");
     }
+    /*  Clean up.
+     */
+    if (hash) {
+        hash_destroy (hash);
+    }
     return;
 }
 
@@ -350,6 +361,7 @@ _gids_hash_create (void)
     hash_t          uid_hash = NULL;
     struct timeval  t_start;
     struct timeval  t_stop;
+    int             do_group_db_close = 0;
     struct group   *gr_ptr;
     char          **pp;
     uid_t           uid;
@@ -375,6 +387,8 @@ _gids_hash_create (void)
         goto err;
     }
     setgrent ();
+    do_group_db_close = 1;
+
     for (;;) {
         errno = 0;
         if (!(gr_ptr = getgrent ())) {
@@ -386,11 +400,12 @@ _gids_hash_create (void)
                 break;
             if (errno == EINTR)
                 continue;
-            log_msg (LOG_ERR, "Unable to query group info");
+            log_msg (LOG_ERR, "Unable to query group info: %s",
+                    strerror (errno));
             goto err;
         }
-        for (pp = gr_ptr->gr_mem; *pp; pp++) {
-            if ((uid = _gids_user_to_uid (uid_hash, *pp)) >= 0) {
+        for (pp = gr_ptr->gr_mem; pp && *pp; pp++) {
+            if ((_gids_user_to_uid (uid_hash, *pp, &uid)) >= 0) {
                 if (_gids_hash_add (gid_hash, uid, gr_ptr->gr_gid) < 0) {
                     goto err;
                 }
@@ -420,7 +435,9 @@ _gids_hash_create (void)
     return (gid_hash);
 
 err:
-    endgrent ();
+    if (do_group_db_close) {
+        endgrent ();
+    }
     hash_destroy (uid_hash);
     hash_destroy (gid_hash);
     return (NULL);
@@ -428,9 +445,10 @@ err:
 
 
 static int
-_gids_user_to_uid (hash_t uid_hash, char *user)
+_gids_user_to_uid (hash_t uid_hash, char *user, uid_t *uid_p)
 {
-/*  Returns the UID associated with [user], or -1 on error.
+/*  Returns 0 on success, setting [*uid_p] (if non-NULL) to the UID associated
+ *    with [user]; o/w, returns -1.
  */
     gids_uid_t     u;
     uid_t          uid;
@@ -459,7 +477,10 @@ _gids_user_to_uid (hash_t uid_hash, char *user)
             "Unable to query password file entry for \"%s\"", user);
         return (-1);
     }
-    return (uid);
+    if (uid_p != NULL) {
+        *uid_p = uid;
+    }
+    return (0);
 }
 
 
@@ -578,10 +599,13 @@ _gids_node_key (uid_t *uid_p)
 static gids_uid_t
 _gids_uid_alloc (char *user, uid_t uid)
 {
-/*  Returns an allocated UID node mapping [user] to [uid].
+/*  Returns an allocated UID node mapping [user] to [uid], or NULL on error.
  */
     gids_uid_t u;
 
+    if ((user == NULL) || (*user == '\0')) {
+        return (NULL);
+    }
     if (!(u = malloc (sizeof (*u)))) {
         return (NULL);
     }
