@@ -1,27 +1,30 @@
 /*****************************************************************************
- *  $Id: munged.c 705 2007-02-04 07:52:37Z dun $
+ *  $Id: munged.c 774 2010-03-10 00:15:47Z dun $
  *****************************************************************************
- *  Copyright (C) 2002-2007 The Regents of the University of California.
- *  Produced at Lawrence Livermore National Laboratory.
  *  Written by Chris Dunlap <cdunlap@llnl.gov>.
+ *  Copyright (C) 2007-2010 Lawrence Livermore National Security, LLC.
+ *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  UCRL-CODE-155910.
  *
  *  This file is part of the MUNGE Uid 'N' Gid Emporium (MUNGE).
  *  For details, see <http://home.gna.org/munge/>.
  *
- *  This is free software; you can redistribute it and/or modify it
- *  under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ *  MUNGE is free software: you can redistribute it and/or modify it under
+ *  the terms of the GNU General Public License as published by the Free
+ *  Software Foundation, either version 3 of the License, or (at your option)
+ *  any later version.  Additionally for the MUNGE library (libmunge), you
+ *  can redistribute it and/or modify it under the terms of the GNU Lesser
+ *  General Public License as published by the Free Software Foundation,
+ *  either version 3 of the License, or (at your option) any later version.
  *
- *  This is distributed in the hope that it will be useful, but WITHOUT
+ *  MUNGE is distributed in the hope that it will be useful, but WITHOUT
  *  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  *  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
- *  for more details.
+ *  and GNU Lesser General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License along
- *  with this program; if not, write to the Free Software Foundation, Inc.,
- *  51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.
+ *  You should have received a copy of the GNU General Public License
+ *  and GNU Lesser General Public License along with MUNGE.  If not, see
+ *  <http://www.gnu.org/licenses/>.
  *****************************************************************************/
 
 
@@ -33,6 +36,7 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <munge.h>
+#include <signal.h>
 #include <stdlib.h>
 #include <string.h>
 #include <sys/time.h>                   /* include before resource.h for bsd */
@@ -56,21 +60,20 @@
 #include "random.h"
 #include "replay.h"
 #include "timer.h"
-#include "version.h"
 
 
 /*****************************************************************************
  *  Prototypes
  *****************************************************************************/
 
+static int  daemonize_init (char *progname);
+static void daemonize_fini (int fd);
+static void open_logfile (const char *logfile, int priority, int got_force);
 static void handle_signals (void);
 static void hup_handler (int signum);
 static void exit_handler (int signum);
 static void segv_handler (int signum);
-static int  daemonize_init (char *progname);
-static void daemonize_fini (int fd);
 static void write_pidfile (const char *pidfile, int got_force);
-static void open_logfile (const char *logfile, int priority, int got_force);
 static void sock_create (conf_t conf);
 static void sock_destroy (conf_t conf);
 
@@ -79,7 +82,7 @@ static void sock_destroy (conf_t conf);
  *  Global Variables
  *****************************************************************************/
 
-int done = 0;                           /* global flag set true for exit     */
+volatile sig_atomic_t done = 0;         /* global flag set true for exit     */
 
 
 /*****************************************************************************
@@ -146,59 +149,6 @@ main (int argc, char *argv[])
 }
 
 
-static void
-handle_signals (void)
-{
-    if (posignal (SIGHUP, hup_handler) == SIG_ERR) {
-        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to handle signal=%d", SIGHUP);
-    }
-    if (posignal (SIGINT, exit_handler) == SIG_ERR) {
-        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to handle signal=%d", SIGINT);
-    }
-    if (posignal (SIGTERM, exit_handler) == SIG_ERR) {
-        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to handle signal=%d", SIGTERM);
-    }
-    if (posignal (SIGSEGV, segv_handler) == SIG_ERR) {
-        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to handle signal=%d", SIGSEGV);
-    }
-    if (posignal (SIGPIPE, SIG_IGN) == SIG_ERR) {
-        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to ignore signal=%d", SIGPIPE);
-    }
-    return;
-}
-
-
-static void
-hup_handler (int signum)
-{
-    if (conf) {
-        gids_update (conf->gids);
-    }
-    return;
-}
-
-
-static void
-exit_handler (int signum)
-{
-    if (!done) {
-        done = 1;
-        log_msg (LOG_NOTICE, "Exiting on signal=%d", signum);
-    }
-    return;
-}
-
-
-static void
-segv_handler (int signum)
-{
-    log_err (EMUNGE_SNAFU, LOG_CRIT,
-        "Exiting on signal=%d (segmentation violation)", signum);
-    assert (1);                         /* not reached */
-    return;
-}
-
-
 static int
 daemonize_init (char *progname)
 {
@@ -347,77 +297,6 @@ daemonize_fini (int fd)
 }
 
 
-static void
-write_pidfile (const char *pidfile, int got_force)
-{
-/*  Creates the specified pidfile.
- *  The pidfile must be created after the daemon has finished forking.
- */
-    char    piddir [PATH_MAX];
-    char    ebuf [1024];
-    int     n;
-    mode_t  mask;
-    FILE   *fp;
-
-    assert (pidfile != NULL);
-
-    /*  The pidfile must be specified with an absolute pathname; o/w, the
-     *    unlink() call in destroy_conf() will fail because the daemon has
-     *    chdir()'d.
-     */
-    if (pidfile[0] != '/') {
-        log_err (EMUNGE_SNAFU, LOG_ERR,
-            "Pidfile \"%s\" requires an absolute path", pidfile);
-    }
-    /*  Ensure pidfile dir is secure against modification by others.
-     */
-    if (path_dirname (pidfile, piddir, sizeof (piddir)) < 0) {
-        log_err (EMUNGE_SNAFU, LOG_ERR,
-            "Cannot determine dirname of pidfile \"%s\"", pidfile);
-    }
-    n = path_is_secure (piddir, ebuf, sizeof (ebuf));
-    if (n < 0) {
-        log_err (EMUNGE_SNAFU, LOG_ERR,
-            "Cannot check pidfile dir \"%s\": %s", piddir, ebuf);
-    }
-    else if ((n == 0) && (!got_force)) {
-        log_err (EMUNGE_SNAFU, LOG_ERR, "Pidfile is insecure: %s", ebuf);
-    }
-    else if (n == 0) {
-        log_msg (LOG_WARNING, "Pidfile is insecure: %s", ebuf);
-    }
-    /*  Protect pidfile against unauthorized access by removing write-access
-     *    from group and other.
-     */
-    mask = umask (0);
-    umask (mask | 022);
-    (void) unlink (pidfile);
-    fp = fopen (pidfile, "w");
-    umask (mask);
-    /*
-     *  An error in creating the pidfile is not considered fatal.
-     */
-    if (!fp) {
-        log_msg (LOG_WARNING, "Unable to open pidfile \"%s\": %s",
-            pidfile, strerror (errno));
-    }
-    else if (fprintf (fp, "%d\n", (int) getpid ()) == EOF) {
-        log_msg (LOG_WARNING, "Unable to write to pidfile \"%s\": %s",
-            pidfile, strerror (errno));
-        (void) fclose (fp);
-    }
-    else if (fclose (fp) == EOF) {
-        log_msg (LOG_WARNING, "Unable to close pidfile \"%s\": %s",
-            pidfile, strerror (errno));
-    }
-    else {
-        return;                         /* success */
-    }
-    (void) unlink (pidfile);
-    return;                             /* failure */
-}
-
-
 static void
 open_logfile (const char *logfile, int priority, int got_force)
 {
@@ -519,6 +398,129 @@ open_logfile (const char *logfile, int priority, int got_force)
 }
 
 
+static void
+handle_signals (void)
+{
+    if (posignal (SIGHUP, hup_handler) == SIG_ERR) {
+        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to handle signal=%d", SIGHUP);
+    }
+    if (posignal (SIGINT, exit_handler) == SIG_ERR) {
+        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to handle signal=%d", SIGINT);
+    }
+    if (posignal (SIGTERM, exit_handler) == SIG_ERR) {
+        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to handle signal=%d", SIGTERM);
+    }
+    if (posignal (SIGSEGV, segv_handler) == SIG_ERR) {
+        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to handle signal=%d", SIGSEGV);
+    }
+    if (posignal (SIGPIPE, SIG_IGN) == SIG_ERR) {
+        log_err (EMUNGE_SNAFU, LOG_ERR, "Unable to ignore signal=%d", SIGPIPE);
+    }
+    return;
+}
+
+
+static void
+hup_handler (int signum)
+{
+    if (conf) {
+        gids_update (conf->gids);
+    }
+    return;
+}
+
+
+static void
+exit_handler (int signum)
+{
+    if (!done) {
+        done = signum;
+    }
+    return;
+}
+
+
+static void
+segv_handler (int signum)
+{
+    log_err (EMUNGE_SNAFU, LOG_CRIT,
+        "Exiting on signal=%d (segmentation violation)", signum);
+    assert (1);                         /* not reached */
+    return;
+}
+
+
+static void
+write_pidfile (const char *pidfile, int got_force)
+{
+/*  Creates the specified pidfile.
+ *  The pidfile must be created after the daemon has finished forking.
+ */
+    char    piddir [PATH_MAX];
+    char    ebuf [1024];
+    int     n;
+    mode_t  mask;
+    FILE   *fp;
+
+    assert (pidfile != NULL);
+
+    /*  The pidfile must be specified with an absolute pathname; o/w, the
+     *    unlink() call in destroy_conf() will fail because the daemon has
+     *    chdir()'d.
+     */
+    if (pidfile[0] != '/') {
+        log_err (EMUNGE_SNAFU, LOG_ERR,
+            "Pidfile \"%s\" requires an absolute path", pidfile);
+    }
+    /*  Ensure pidfile dir is secure against modification by others.
+     */
+    if (path_dirname (pidfile, piddir, sizeof (piddir)) < 0) {
+        log_err (EMUNGE_SNAFU, LOG_ERR,
+            "Cannot determine dirname of pidfile \"%s\"", pidfile);
+    }
+    n = path_is_secure (piddir, ebuf, sizeof (ebuf));
+    if (n < 0) {
+        log_err (EMUNGE_SNAFU, LOG_ERR,
+            "Cannot check pidfile dir \"%s\": %s", piddir, ebuf);
+    }
+    else if ((n == 0) && (!got_force)) {
+        log_err (EMUNGE_SNAFU, LOG_ERR, "Pidfile is insecure: %s", ebuf);
+    }
+    else if (n == 0) {
+        log_msg (LOG_WARNING, "Pidfile is insecure: %s", ebuf);
+    }
+    /*  Protect pidfile against unauthorized access by removing write-access
+     *    from group and other.
+     */
+    mask = umask (0);
+    umask (mask | 022);
+    (void) unlink (pidfile);
+    fp = fopen (pidfile, "w");
+    umask (mask);
+    /*
+     *  An error in creating the pidfile is not considered fatal.
+     */
+    if (!fp) {
+        log_msg (LOG_WARNING, "Unable to open pidfile \"%s\": %s",
+            pidfile, strerror (errno));
+    }
+    else if (fprintf (fp, "%d\n", (int) getpid ()) == EOF) {
+        log_msg (LOG_WARNING, "Unable to write to pidfile \"%s\": %s",
+            pidfile, strerror (errno));
+        (void) fclose (fp);
+    }
+    else if (fclose (fp) == EOF) {
+        log_msg (LOG_WARNING, "Unable to close pidfile \"%s\": %s",
+            pidfile, strerror (errno));
+    }
+    else {
+        return;                         /* success */
+    }
+    (void) unlink (pidfile);
+    return;                             /* failure */
+}
+
+
 static void
 sock_create (conf_t conf)
 {
